'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _events = require('events');

var _events2 = _interopRequireDefault(_events);

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _onUpdateReceived = require('./onUpdateReceived');

var _onUpdateReceived2 = _interopRequireDefault(_onUpdateReceived);

var _getLastUpdateId = require('./getLastUpdateId');

var _getLastUpdateId2 = _interopRequireDefault(_getLastUpdateId);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new _bluebird2.default(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return _bluebird2.default.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

// eslint-disable-next-line flowtype/no-weak-types


// eslint-disable-next-line flowtype/no-weak-types
class BotClient extends _events2.default {

  constructor(token, sendRequest) {
    var _this;

    _this = super();

    this.lastUpdateId = 0;
    this.watchUpdatesInterval = null;
    this.pullUpdates = _asyncToGenerator(function* () {
      try {
        _this.publishUpdates((yield _this.getUpdates()));
      } catch (error) {
        _this.emit('getUpdatesError', error);
      }
    });
    Object.defineProperties(this, {
      token: { value: token, enumerable: true },
      sendRequest: { value: sendRequest, enumerable: true }
    });

    this.on('updateReceived', _onUpdateReceived2.default);
  }

  createReaction(timeout = 1000 * 60 * 5) {
    return predicate => {
      let listener;

      const unsubscribe = () => this.removeListener('updateReceived', listener);

      const promise = new _bluebird2.default(resolve => {
        listener = update => predicate(update) && resolve(update);
        this.on('updateReceived', listener);
      });

      return (timeout ? promise.timeout(timeout, 'timeout').catch(error => {
        unsubscribe();return _bluebird2.default.reject(error);
      }) : promise).then(update => {
        unsubscribe();return update;
      });
    };
  }

  callMethod(methodName, args) {
    var _this2 = this;

    return _asyncToGenerator(function* () {
      const response = yield _this2.sendRequest(`https://api.telegram.org/bot${_this2.token}/${methodName}`, args);

      if (!response.ok) throw response;

      return response.result;
    })();
  }

  lastUpdateIdRefresh(updates) {
    if (updates.length) this.lastUpdateId = (0, _getLastUpdateId2.default)(updates);
  }

  publishUpdates(updates) {
    updates.forEach(update => this.emit('updateReceived', update));
  }

  startWatchUpdates(interval = 5) {
    this.stopWatchUpdates();
    this.watchUpdatesInterval = setInterval(this.pullUpdates, interval * 1000);
  }

  stopWatchUpdates() {
    if (!this.watchUpdatesInterval) return;

    clearInterval(this.watchUpdatesInterval);
    this.watchUpdatesInterval = null;
  }

  getUpdates(_ref2 = {}) {
    var _this3 = this;

    let {
      offset = this.lastUpdateId + 1 } = _ref2,
        args = _objectWithoutProperties(_ref2, ['offset']);

    return _asyncToGenerator(function* () {
      const response = yield _this3.callMethod('getUpdates', _extends({}, args, { offset }));

      _this3.lastUpdateIdRefresh(response);
      return response;
    })();
  }

  setWebhook(args) {
    return this.callMethod('setWebhook', args);
  }

  deleteWebhook() {
    return this.callMethod('deleteWebhook');
  }

  getWebhookInfo() {
    return this.callMethod('getWebhookInfo');
  }

  getMe() {
    return this.callMethod('getMe');
  }

  sendMessage(args) {
    return this.callMethod('sendMessage', args);
  }

  forwardMessage(args) {
    return this.callMethod('forwardMessage', args);
  }

  sendPhoto(args) {
    return this.callMethod('sendPhoto', args);
  }

  sendAudio(args) {
    return this.callMethod('sendAudio', args);
  }

  sendDocument(args) {
    return this.callMethod('sendDocument', args);
  }

  sendSticker(args) {
    return this.callMethod('sendSticker', args);
  }

  sendVideo(args) {
    return this.callMethod('sendVideo', args);
  }

  sendVoice(args) {
    return this.callMethod('sendVoice', args);
  }

  sendLocation(args) {
    return this.callMethod('sendLocation', args);
  }

  sendVenue(args) {
    return this.callMethod('sendVenue', args);
  }

  sendContact(args) {
    return this.callMethod('sendContact', args);
  }

  sendChatAction(args) {
    return this.callMethod('sendChatAction', args);
  }

  getUserProfilePhotos(args) {
    return this.callMethod('getUserProfilePhotos', args);
  }

  getFile(args) {
    return this.callMethod('getFile', args);
  }

  kickChatMember(args) {
    return this.callMethod('kickChatMember', args);
  }

  leaveChat(args) {
    return this.callMethod('leaveChat', args);
  }

  unbanChatMember(args) {
    return this.callMethod('unbanChatMember', args);
  }

  getChat(args) {
    return this.callMethod('getChat', args);
  }

  getChatAdministrators(args) {
    return this.callMethod('getChatAdministrators', args);
  }

  getChatMembersCount(args) {
    return this.callMethod('getChatMembersCount', args);
  }

  getChatMember(args) {
    return this.callMethod('getChatMember', args);
  }

  answerCallbackQuery(args) {
    return this.callMethod('answerCallbackQuery', args);
  }

  editMessageText(args) {
    return this.callMethod('editMessageText', args);
  }

  editMessageCaption(args) {
    return this.callMethod('editMessageCaption', args);
  }

  editMessageReplyMarkup(args) {
    return this.callMethod('editMessageReplyMarkup', args);
  }
}

exports.default = BotClient;