'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getPressedInlineButtonId = exports.getCallbackQueryDataJSON = exports.getCallbackQueryData = exports.getCallbackQuery = exports.getPre = exports.getCode = exports.getItalic = exports.getBold = exports.getEmails = exports.getUrls = exports.getBotCommands = exports.getHashtags = exports.getMentions = exports.getTextLinks = exports.getTextMentions = exports.getMessage = exports.getUpdateType = undefined;

var _reselect = require('reselect');

const updateIdFilter = key => key !== 'update_id';

const getUpdateDataKey = update => {
  const keys = Object.keys(update);
  const filtered = keys.filter(updateIdFilter);
  return filtered[0];
};

const getUpdateType = exports.getUpdateType = (0, _reselect.createSelector)(getUpdateDataKey, type => type);

const getMessage = exports.getMessage = ({ message }) => message;

const getMessageText = (0, _reselect.createSelector)(getMessage, ({ text } = {}) => text);

const getMessageRawEntities = (0, _reselect.createSelector)(getMessage, message => message && message.entities ? message.entities : []);

const botCommandArgsRegExp = /^([^\n/#@])[\n/#@]|$/g;

const entitiesReduser = messageText => (entities, entity) => {
  if (typeof messageText !== 'string') return entities;

  const text = messageText.substr(entity.offset, entity.length);

  switch (entity.type) {
    case 'text_mention':
      entities.textMentions.push([text, entity.user]);
      break;
    case 'text_link':
      entities.textLinks.push([text, entity.url]);
      break;
    case 'mention':
      entities.mentions.push(text);
      break;
    case 'hashtag':
      entities.hashtags.push(text);
      break;
    case 'bot_command':
      {
        const botCommandArgs = messageText.substr(entity.offset + entity.length).replace(botCommandArgsRegExp, '$1').trim();

        entities.botCommands.push([text, botCommandArgs]);
        break;
      }
    case 'url':
      entities.urls.push(text);
      break;
    case 'email':
      entities.emails.push(text);
      break;
    case 'bold':
      entities.bold.push(text);
      break;
    case 'italic':
      entities.italic.push(text);
      break;
    case 'code':
      entities.code.push(text);
      break;
    case 'pre':
      entities.pre.push(text);
      break;
    default:
      // eslint-disable-next-line no-unused-expressions
      entity;
  }

  return entities;
};

const initEntitiesScope = () => ({
  textMentions: [],
  textLinks: [],
  mentions: [],
  hashtags: [],
  botCommands: [],
  urls: [],
  emails: [],
  bold: [],
  italic: [],
  code: [],
  pre: []
});

const getMessageEntities = (0, _reselect.createSelector)(getMessageText, getMessageRawEntities, (messageText, rawEntities) => rawEntities.reduce(entitiesReduser(messageText), initEntitiesScope()));

const getTextMentions = exports.getTextMentions = (0, _reselect.createSelector)(getMessageEntities, ({ textMentions }) => textMentions);
const getTextLinks = exports.getTextLinks = (0, _reselect.createSelector)(getMessageEntities, ({ textLinks }) => textLinks);
const getMentions = exports.getMentions = (0, _reselect.createSelector)(getMessageEntities, ({ mentions }) => mentions);
const getHashtags = exports.getHashtags = (0, _reselect.createSelector)(getMessageEntities, ({ hashtags }) => hashtags);
const getBotCommands = exports.getBotCommands = (0, _reselect.createSelector)(getMessageEntities, ({ botCommands }) => botCommands);
const getUrls = exports.getUrls = (0, _reselect.createSelector)(getMessageEntities, ({ urls }) => urls);
const getEmails = exports.getEmails = (0, _reselect.createSelector)(getMessageEntities, ({ emails }) => emails);
const getBold = exports.getBold = (0, _reselect.createSelector)(getMessageEntities, ({ bold }) => bold);
const getItalic = exports.getItalic = (0, _reselect.createSelector)(getMessageEntities, ({ italic }) => italic);
const getCode = exports.getCode = (0, _reselect.createSelector)(getMessageEntities, ({ code }) => code);
const getPre = exports.getPre = (0, _reselect.createSelector)(getMessageEntities, ({ pre }) => pre);

const getCallbackQuery = exports.getCallbackQuery = ({ callback_query: callbackQuery }) => callbackQuery;

const getCallbackQueryData = exports.getCallbackQueryData = (0, _reselect.createSelector)(getCallbackQuery, ({ data } = {}) => data);

const getCallbackQueryDataJSON = exports.getCallbackQueryDataJSON = (0, _reselect.createSelector)(getCallbackQueryData, data => {
  try {
    // eslint-disable-next-line flowtype/no-weak-types
    return JSON.parse(data);
  } catch (e) {
    return undefined;
  }
});

const getPressedInlineButtonId = exports.getPressedInlineButtonId = (0, _reselect.createSelector)(getCallbackQueryDataJSON, data => data && (typeof data.pressedInlineButtonId === 'string' || typeof data.pressedInlineButtonId === 'number') ? data.pressedInlineButtonId : undefined);